% This file demonstrates how a Kalman filter can be used to estimate the
% speed of a car. Try changing w0 (and zeta).
close all
clc

% system dynamics (at speed 10 m/s)
A=[0 1;0 0]; % x1=position, x2=speed
B=[0;1]; % u=acceleration proportional to throttle 
C=[1 0]; % y=position measured by GPS
D=0;
P=ss(A,B,C,D);

% disturbances
d=.2; % offset on u, due to sloping road
n=1e-8; % noise on y, caused by GPS receiver

% observer design
w0=10; % observer speed (noise sensitivity). Try e.g. 0.1 1 10 100
zeta=.7; % relative damping
poles=roots([1 2*zeta*w0 w0^2]);
K=acker(A',C',poles)'; % pole placement

% simulate
sim('carModel')
t=simout.Time;
simout.Data=simout.Data+10; % offset to equilibrium point (10 m/s)
x1=simout.Data(:,1);
y1=simout.Data(:,2);
y2_int=simout.Data(:,3);
x1_hat=simout.Data(:,4);

% plot
figure(1)
plot(t,x1,'b','Linewidth',2)
ylim([9 14])
hold on
plot(t,y2_int,'m','Linewidth',2)
plot(t,y1,'g','Linewidth',2)
plot(t,x1,'b','Linewidth',2)
plot(t,x1_hat,'r','Linewidth',2)
hold off